@@ -9,11 +9,7 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 2. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by Eric Youngdale,
- *      Peter MacDonald, David Engel and Hongjiu Lu.
- * 3. The name of the above contributors may not be
+ * 2. The name of the above contributors may not be
  *    used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
