@@ -8,17 +8,25 @@
 #  define LDSO_CACHE "/etc/ld.so.cache"
 #endif
 
-#define VERSION      "1.4.4"
+#define VERSION      "1.5.3"
 #define LDD_ARGV0    "__LDD_ARGV0"
 #define DIR_SEP      ":, \t\n"
 #define MAX_DIRS     32
 #define DEFAULT_PATH "/usr/lib:/lib"
 
 typedef void (*loadptr)(int func, ...);
+typedef void (*callbackptr)(int ver, int nlibs, char **libs, 
+		int nmods, char **mods);
+
+#define CALLBACK_VER 1
+
+#define LIB_DLL      0
+#define LIB_ELF      1
 
 #define FUNC_VERS    0
 #define FUNC_LDD     1
 #define FUNC_LINK    2
+#define FUNC_LINK_AND_CALLBACK 3
 
 #define LDSO_CACHE_MAGIC "ld.so-"
 #define LDSO_CACHE_MAGIC_LEN (sizeof LDSO_CACHE_MAGIC -1)
@@ -35,6 +43,7 @@ typedef struct {
 } header_t;
 
 typedef struct {
+	char flags;
 	short libnamelen;
 	short dirnamelen;
 	long liboffset;
