@@ -8,7 +8,7 @@ if [ -e /etc/ld.so.cache ] ; then
 fi
 
 if [ ! -e /etc/ld.so.conf ] ; then
-	for dir in /usr/local/lib /usr/openwin/lib /usr/X386/lib ; do
+	for dir in /usr/local/lib /usr/X11R6/lib /usr/X386/lib /usr/openwin/lib /lib/elf ; do
 		if [ -d $dir ] ; then
 			echo Adding $dir to /etc/ld.so.conf
 			echo $dir >> /etc/ld.so.conf
