@@ -9,11 +9,7 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- * 2. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by Eric Youngdale,
- *      Peter MacDonald, David Engel and Hongjiu Lu.
- * 3. The name of the above contributors may not be
+ * 2. The name of the above contributors may not be
  *    used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -63,7 +59,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <limits.h>
-#include <string.h>
+#include <asm/string.h>
 #include <sys/stat.h>
 #include "config.h"
 #include "fixups.h"
@@ -83,7 +79,7 @@
 
 int errno = 0;
 char **__environ = 0;
-
+char *___strtok=NULL;
 extern int _entry;
 extern int _edata;
 
@@ -146,11 +142,7 @@ save_mapinfo(unsigned *mapinfo)
 {
     if (mapinfo)
     {
-#ifdef LDSO_QMAGIC
 	mapinfo[0] = (unsigned)&_entry-0x20;
-#else
-	mapinfo[0] = (unsigned)&_entry;
-#endif
 	mapinfo[1] = ((unsigned)&_edata-(unsigned)&_entry+0xfff) & ~0xfff;
     }
 }
@@ -172,11 +164,14 @@ shared_loader(int func, ...)
 #ifndef DEBUG_LIB
     unsigned *mapinfo = NULL;
 #endif
-
+    char *libs[10];
+    int nlibs=0;
     int nowarn;
     int keepdir;
+    char *preload;
     caddr_t cache_addr;
     size_t cache_size;
+    callbackptr callback = (callbackptr)0;
 
 #ifdef TIMER
 #include <time.h>
@@ -201,6 +196,7 @@ shared_loader(int func, ...)
 	/* just going through the motions */
 	ldd = 1;
 	/* fall through */;
+    case FUNC_LINK_AND_CALLBACK:
     case FUNC_LINK:
 	/* this is why we're all here */
 	va_start(ap, func);
@@ -212,6 +208,8 @@ shared_loader(int func, ...)
 	__environ = va_arg(ap, char **);
 	__SHARED_LIBRARIES__ = va_arg(ap, struct libentry **);
 	_SHARABLE_CONFLICTS__ = va_arg(ap, struct fixuplist *);
+	if (func == FUNC_LINK_AND_CALLBACK)
+	  callback = va_arg(ap, callbackptr);
 	va_end(ap);
 	break;
     default:
@@ -229,13 +227,14 @@ shared_loader(int func, ...)
     if (!argv0 && !(argv0 = getenv(LDD_ARGV0)))
 	argv0 = LDSO_IMAGE;
 
-    /* so you want your own path, do you? */
-    if ((cp = getenv("LD_LIBRARY_PATH")) && *cp) 
+    /* hmm, you want your own path, do you? */
+    if (((cp = getenv("AOUT_LD_LIBRARY_PATH")) && *cp) ||
+        ((cp = getenv("LD_LIBRARY_PATH")) && *cp))
     {
 	uid_t uid = getuid();
 	if (uid && (uid != geteuid() || getgid() != getegid()))
 	{
-	    /* sorry Charlie, I can't let you do that */
+	    /* sorry, Charlie, I can't let you do that */
 	    *cp = '\0';
 	}
 	else
@@ -246,6 +245,17 @@ shared_loader(int func, ...)
 	}
     }
 
+    /* so you want to override something, eh? */
+    if ((preload = getenv("LD_PRELOAD")))
+    {
+	uid_t uid = getuid();
+	if (uid && (uid != geteuid() || getgid() != getegid()))
+	{
+	    /* no way, Jose, that's not allowed here */
+	    *preload = '\0';
+	}
+    }
+
     /* If the cache exists then we don't add these */
     if (!(cache_addr = cache_init(LDSO_CACHE, &cache_size))) 
     {
@@ -304,6 +314,7 @@ shared_loader(int func, ...)
 		for (i = 0; !found && i < header->nlibs; i++)
 		{
 		    if (libnamelen == (libent[i].libnamelen >> 8) &&
+			libent[i].flags == LIB_DLL &&
 			!memcmp(cp, cache_addr + header->liboffset +
 				libent[i].liboffset, libnamelen)) 
 			found = try_lib(argv0, buffer, 
@@ -339,6 +350,15 @@ shared_loader(int func, ...)
 	}
 	else
 	{
+	     /* for preloading we need to know the offsets and length of the
+		.text section. This is passed to ldpreload() below */
+	    if (preload || callback)
+	    {
+	        libs[nlibs] = alloca(strlen(buffer)+1);
+	        strcpy(libs[nlibs], buffer);
+	        nlibs++;
+	    }
+
 	    /* hey, I think I've got a live one here */
 	    if (ldd)
 	    {
@@ -414,6 +434,10 @@ shared_loader(int func, ...)
 	    exit(EXIT_FATAL);
 	}
     }
+    if (preload)
+      ldpreload(preload, ldd, libs, nlibs, argv0);
+    if (callback)
+      (*callback)(CALLBACK_VER, nlibs, libs, preload ? 1 : 0, &preload);
 }
 
 static caddr_t cache_init(char *file, size_t *size)
