@@ -23,12 +23,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <string.h>
+#include <asm/string.h>
 #include <ctype.h>
 #include <getopt.h>
 #include <dirent.h>
 #include <unistd.h>
 #include <a.out.h>
+#include <linux/elf.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
@@ -36,6 +37,8 @@
 
 #include "config.h"
 
+char *___strtok = NULL;
+
 /* For SunOS */
 #ifndef PATH_MAX
 #include <limits.h>
@@ -67,7 +70,7 @@ signed char invert = 1;		/* Invert any of the sort options */
 
 void cache_print(void);
 void cache_dodir(char *, short);
-void cache_dolib(char *, short);
+void cache_dolib(char *, short, char);
 void cache_write(void);
 
 void warn(char *fmt, ...)
@@ -111,23 +114,25 @@ void *xmalloc(size_t size)
 char *xstrdup(char *str)
 {
     char *ptr;
+    char *strdup(const char *);
     if ((ptr = strdup(str)) == NULL)
 	error("out of memory");
     return ptr;
 }
 
 /* if shared library, return length of base name, else return -1 */
-int is_shlib(char *dir, char *name)
+int is_shlib(char *dir, char *name, char *type)
 {
     int good = 0;
     char *cp = name;
     FILE *file;
     struct exec exec;
+    struct elfhdr *elf_hdr;
     char buff[1024];
 
     /* see if name is of the form libZ.so.M.N[.P] */
-    if (strncmp(name, "lib", 3) == 0 && (cp = strstr(name, ".so."))
-	&& (cp = strchr(cp + 4, '.')))
+    if ((strncmp(name, "lib", 3) == 0 || strncmp(name, "ld-", 3) == 0)
+	&& (cp = strstr(name, ".so.")) && (cp = strchr(cp + 4, '.')))
     {
 	/* construct the full path name */
 	sprintf(buff, "%s%s%s", dir, (*dir && strcmp(dir, "/")) ?
@@ -138,16 +143,28 @@ int is_shlib(char *dir, char *name)
 	    warn("can't open %s (%s), skipping", buff, strerror(errno));
 	else
 	{
-	    /* now make sure it's ZMAGIC or QMAGIC with a non-zero address */
+	    /* now make sure it's [ZQ]MAGIC with a non-zero address or ELF */
 	    if (fread(&exec, sizeof exec, 1, file) < 1)
 		warn("can't read header from %s, skipping", buff);
 	    else if (N_MAGIC(exec) != ZMAGIC && N_MAGIC(exec) != QMAGIC)
-		warn("%s is not a ZMAGIC file, skipping", buff);
+	    {
+		elf_hdr = (struct elfhdr *) &exec;
+		if (elf_hdr->e_ident[0] != 0x7f ||
+			 strncmp(&elf_hdr->e_ident[1], "ELF",3) != 0)
+		    warn("%s is not a [ZQ]MAGIC or ELF file, skipping", buff);
+		else
+		{
+		    good = 1;
+		    *type = LIB_ELF;
+		}
+	    }
 	    else if (!exec.a_entry)
 		warn("%s has a zero load address, skipping", buff);
 	    else
+	    {
 		good = 1;	/* looks good */
-
+		*type = LIB_DLL;
+	    }
 	    fclose(file);
 	}
     }
@@ -238,6 +255,7 @@ void scan_dir(char *name)
     struct dirent *ent;
     int len;
     struct lib *lp, *libs = NULL;
+    char libtype;
 
     /* let 'em know what's going on */
     if (verbose)
@@ -257,11 +275,11 @@ void scan_dir(char *name)
     while ((ent = readdir(dir)) != NULL)
     {
 	/* if it's not a shared library, don't bother */
-	if ((len = is_shlib(name, ent->d_name)) < 0)
+	if ((len = is_shlib(name, ent->d_name, &libtype)) < 0)
 	    continue;
 
 	if (!nocache)
-	    cache_dolib(ent->d_name, len);
+	    cache_dolib(ent->d_name, len, libtype);
 
 	/* have we already seen one with the same base name? */
 	for (lp = libs; lp; lp = lp->next)
@@ -334,7 +352,7 @@ int main(int argc, char **argv)
     int nodefault = 0;
     int printcache = 0;
     char *cp, *dir;
-    char *extpath;
+    char *extpath, libtype;
     static char defpath[] = DEFAULT_PATH;
 
     prog = argv[0];
@@ -437,7 +455,7 @@ int main(int argc, char **argv)
 	    }
 
 	    /* we'd better do a little bit of checking */
-	    if ((len = is_shlib(dir, cp)) < 0)
+	    if ((len = is_shlib(dir, cp, &libtype)) < 0)
 		error("%s%s%s is not a shared library", dir,
 		      (*dir && strcmp(dir, "/")) ? "/" : "", cp);
 
@@ -496,6 +514,7 @@ typedef struct dirlist
 
 typedef struct liblist
 {
+    char flags;
     short libnamelen;
     short dirnamelen;
     long liboffset;
@@ -537,7 +556,7 @@ void cache_dodir(char *entry, short len)
     }
 }
 
-void cache_dolib(char *entry, short len)
+void cache_dolib(char *entry, short len, char libtype)
 {
     char fullpath[PATH_MAX];
 
@@ -546,6 +565,7 @@ void cache_dolib(char *entry, short len)
     {
 	sprintf(fullpath, "%s/%s", cur_dir->entry, entry);
 	cur_lib = (liblist_t *) xmalloc(sizeof (liblist_t));
+	cur_lib->flags = libtype;
 	cur_lib->libnamelen = strlen(entry)+(len<<8);
 	cur_lib->dirnamelen = cur_dir->dirnamelen;
 	cur_lib->diroffset = cur_dir->offset;
@@ -559,6 +579,7 @@ void cache_dolib(char *entry, short len)
     {
 	sprintf(fullpath, "%s/%s", cur_dir->entry, entry);
 	cur_lib->q_forw = (liblist_t *) xmalloc(sizeof (liblist_t));
+	cur_lib->q_forw->flags = libtype;
 	cur_lib->q_forw->libnamelen = strlen(entry)+(len<<8);
 	cur_lib->q_forw->dirnamelen = cur_dir->dirnamelen;
 	cur_lib->q_forw->diroffset = cur_dir->offset;
@@ -684,6 +705,7 @@ void cache_print(void)
 	write(2,diroffset+libent.diroffset,libent.dirnamelen);
 	write(2,"/",1);
 	write(2,liboffset+libent.liboffset,libent.libnamelen & LINK_MASK);
+	write(2, libent.flags == LIB_DLL ? " (DLL)":" (ELF)",6);
 	write(2,"\n",1);
     }
     munmap (c,st.st_size);
